#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-


# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# If you have not received a copy of the GNU General Public License along
# with this program, see <https://www.gnu.org/licenses/>.


# part of mkp package "powerscale"
# see ~/local/share/doc/check_mk/powerscale for maintainer and information

#<<<powerscale_system>>>
#cpu 4.1
#disk_in 71855.54285714286
#disk_out 0.0
#ftp 0.0
#hdfs 0.0
#http 452.8328247070312
#net_in 214.2666666666667
#net_out 43.6
#nfs 0.0
#node All
#s3 0.0
#smb 0.0
#time 1680766015
#total 452.8328247070312



# import api
from .agent_based_api.v1 import (
    Metric,
    register,
    render,
    Result,
    State,
    Service,
)

# factory settings:
powerscale_system_factory_settings = {
    'upper_levels_cpu'  :  (0.0, 0.0),
    'upper_levels_net'  :  (0, 0),
    'upper_levels_disk' :  (0, 0),
    'upper_levels_ftp'  :  (0, 0),
    'upper_levels_hdfs' :  (0, 0),
    'upper_levels_http' :  (0, 0),
    'upper_levels_nfs'  :  (0, 0),
    'upper_levels_s3'   :  (0, 0),
    'upper_levels_smb'  :  (0, 0),
}

# parse function
def parse_powerscale_system(string_table):

    # we'll return this later
    data              = {}

    # loop thru
    for line in string_table:
        try:
            data[line[0]] = float(line[1])
        except:
            pass

    return data

# discovery function
def discover_powerscale_system(section):

    if len(section) > 0:
        yield Service()

# check function
def check_powerscale_system(params, section):


    # initial
    mysummary    = []
    mydetails    = []
    mydetails.append('System Time: %s' % render.datetime(section['time']))

    # CPU
    warn, crit = params['upper_levels_cpu']
    c          = section['cpu']
    marker = '(!!)' if c > crit and crit > 0 else '(!)'if c > warn and crit > 0 else ''
    mysummary.append('CPU: %d%%%s' % (c, marker))
    yield Metric('cpu_percent', c, levels=(warn,crit))

    # main bandwidth
    for what in ['net', 'disk']:
        warn, crit = params['upper_levels_%s' % what]
        myin       = section['%s_in'  % what]
        myout      = section['%s_out' % what]
        val        = max(myin, myout)
        marker     = '(!!)' if val > crit and crit > 0 else '(!)'if val > warn and crit > 0 else ''
        if what in ['net']:
            hrin = render.networkbandwidth(myin)
            hrout = render.networkbandwidth(myout)
            yield Metric('if_in_bps',  myin*8,  levels=(warn,crit))
            yield Metric('if_out_bps', myout*8, levels=(warn,crit))
            
        else:
            hrin = render.iobandwidth(myin)
            hrout = render.iobandwidth(myout)
            yield Metric('disk_write_throughput', myin,  levels=(warn,crit))
            yield Metric('disk_read_throughput',  myout, levels=(warn,crit))

        mysummary.append('%s In/Out: %s/%s%s' % (what.capitalize(), hrin, hrout, marker))


    # services
    for what in ['ftp', 'hdfs', 'http', 'nfs', 's3', 'smb']:
        warn, crit = params['upper_levels_%s' % what]
        w          = section[what]
        marker     = '(!!)' if w > crit and crit > 0 else '(!)'if w > warn and crit > 0 else ''
        if '!' in marker:
            mysummary.append('%s: %s%s (above %s/%s)' % (
                              what, 
                              render.networkbandwidth(section[what]), 
                              marker, 
                              render.networkbandwidth(warn), 
                              render.networkbandwidth(crit)))
        else:
            mydetails.append('%s: %s' % (what, render.networkbandwidth(section[what])))

        yield Metric('bytes_%s' % what, int(section[what]), levels=(warn,crit))
        

    # get worst state from markers
    mysummary = ', '.join(mysummary) if len(mysummary) > 0 else 'Hardware Status OK'
    mydetails = '\n'.join(mydetails)
    mystate   = State.CRIT if '!!' in mysummary else State.WARN if '!' in mysummary else State.OK

    # this is the end, my friend
    yield Result(state=mystate, summary=mysummary, details=mydetails)
    return

        

# register parse function
register.agent_section(
    name                = "powerscale_system",
    parse_function      = parse_powerscale_system,
)

# register check
register.check_plugin(
    name                     = "powerscale_system",
    service_name             = "PowerScale Statistics",
    discovery_function       = discover_powerscale_system,
    check_function           = check_powerscale_system,
    check_ruleset_name       = "powerscale_system",
    check_default_parameters = powerscale_system_factory_settings,
)
